export class Color extends HTMLElement {
  constructor() {
    super();
    const shadow = this.attachShadow({ mode: 'open' });
    shadow.innerHTML = `
      <div class="row">
        <div class="color"></div>
        <div  class="prop"></div>
        <div class="description"></div>
        <div class="css-props"></div>
        <div class="group"></div>
      </div>
      <style>
        :host {
          font-family: sans-serif;
          font-size: 0.8125rem;
          line-height: 1.35;
          flex-direction: column;
          align-items: center;
          flex: 1 1 auto;
        }
        .row {
          display: grid;
          grid-template-columns: repeat(auto-fill, 250px);
          grid-auto-flow: column;
          gap: 1rem;
          align-items: center;
          position: relative;
        }
        .row > div {
          position: relative;
        }
        .row > div + div:after {
          content: '';
          height: 100%;
          width: 1px;
          background: rgba(0,0,0,.1);
          position: absolute;
          right: calc(-0.5rem + 1px);
          top: 0;
        }
        .row:after {
          content: '';
          height: 1px;
          width: 100%;
          background: rgba(0,0,0,.1);
          position: absolute;
          top: 0;
        }
        .color {
          display: flex;
          flex-direction: column;
          padding: 1.5rem 1rem;
          justify-content: center;
          align-content: center;
          font-weight: bold;
          -webkit-font-smoothing: antialiased;
          -moz-osx-font-smoothing: grayscale;
          position: relative;
          z-index: 1;
          border-right: 1px solid rgba(0,0,0,.1);
        }
        code {
          padding: 0.2em 0.4em;
          margin: 0;
          background-color: rgba(27, 31, 35, 0.05);
          border-radius: 6px;
          white-space: pre-wrap;
        }
        p {
          margin: 0;
        }
      </style>
    `;
  }

  get value() {
    return this.getAttribute('value');
  }

  get prop() {
    return this.getAttribute('prop');
  }

  get title() {
    return this.getAttribute('title');
  }

  get group() {
    return this.getAttribute('group');
  }

  get cssProperty() {
    return this.getAttribute('cssProperty');
  }

  _getContrast(c) {
    c = c.replace('#', '');
    var r = parseInt(c.substr(0, 2), 16);
    var g = parseInt(c.substr(2, 2), 16);
    var b = parseInt(c.substr(4, 2), 16);
    var yiq = (r * 299 + g * 587 + b * 114) / 1000;
    return yiq >= 128 ? 'black' : 'white';
  }

  _checkContrast(el, value) {
    if (this._getContrast(value) === 'black') {
      el.style.setProperty('color', 'rgba(0,0,0,0.9)');
    } else {
      el.style.setProperty('color', 'rgba(255,255,255,0.9)');
    }
  }

  connectedCallback() {
    const color = this.shadowRoot.querySelector('.color');
    const prop = this.shadowRoot.querySelector('.prop');
    const description = this.shadowRoot.querySelector('.description');
    const group = this.shadowRoot.querySelector('.group');
    const cssProps = this.shadowRoot.querySelector('.css-props');

    if (this.hasAttribute('value')) {
      color.setAttribute('style', `background-color: ${this.value};`);
      color.insertAdjacentHTML('beforeend', `<code>${this.value}</code>`);
    }
    if (this.hasAttribute('prop')) {
      prop.insertAdjacentHTML('beforeend', `<code>${this.prop}</code>`);
    }
    if (this.group !== 'undefined') {
      group.innerHTML = `<p>${this.group}</p>`;
    } else {
      group.remove();
    }
    description.innerHTML = `<p>${this.title}</p>`;
    cssProps.innerHTML = `<code>${this.cssProperty}</code>`;

    this._checkContrast(color, this.value);
  }
}

export class ColorHeader extends HTMLElement {
  constructor() {
    super();
    const shadow = this.attachShadow({ mode: 'open' });
    shadow.innerHTML = `
      <div class="row">
        <div class="color">Value</div>
        <div  class="prop">CSS custom property</div>
        <div class="description">Description</div>
        <div class="css-props">CSS properties</div>
        <div class="group">Group</div>
      </div>
      <style>
        :host {
          font-family: sans-serif;
          font-size: 0.8125rem;
          flex-direction: column;
          align-items: center;
        }
        .row {
          display: grid;
          grid-template-columns: repeat(auto-fill, 250px);
          grid-auto-flow: column;
          gap: 1rem;
          font-weight: bold;
        }
        .row > div {
          padding: 0.25rem;
        }
      </style>
    `;
  }
}
