const theo = require('theo');
const path = require('path');
const fs = require('fs-extra');
const glob = require('glob');

// Register custom formats
require('../../src/formats/custom-props');
require('../../src/formats/register-props');

const root = path.resolve(__dirname, '../../');
const buildDir = path.resolve(root, '.build/');

const formats = [
  {
    format: 'json',
    transform: 'web',
  },
  {
    format: 'common.js',
    transform: 'web',
  },
  {
    format: 'module.js',
    transform: 'web',
  },
  {
    format: 'custom-props.css',
    transform: 'raw',
  },
  {
    format: 'raw.json',
    transform: 'raw',
  },
  {
    format: 'register-props.js',
    transform: 'raw',
  },
];

glob(path.resolve(root, 'src/props/*.json'), (err, data) => {
  if (err) throw err;
  data.map((file) => {
    const name = path.parse(file).name;
    formats.map((type) => {
      theo
        .convert({
          transform: {
            type: type.transform,
            file: file,
          },
          format: {
            type: type.format,
          },
          resolveMetaAliases: true,
        })
        .then((data) => {
          fs.outputFile(path.resolve(buildDir, `${name}.${type.format}`), data, (err) => {
            if (err) return console.error(err);
          });
        })
        .catch((error) => console.error(`Something went wrong: ${error} in ${file}`));
    });
  });
});

// Transform files for documentation
glob(path.resolve(root, 'src/props/shared/*.json'), (err, data) => {
  if (err) throw err;
  data.map((file) => {
    const name = path.parse(file).name;
    theo
      .convert({
        transform: {
          type: 'raw',
          file: file,
        },
        format: {
          type: 'raw.json',
        },
        resolveMetaAliases: true,
      })
      .then((data) => {
        fs.outputFile(path.resolve(buildDir, `${name}.raw.json`), data, (err) => {
          if (err) return console.error(err);
        });
      })
      .catch((error) => console.error(`Something went wrong: ${error} in ${file}`));
  });
});
